﻿package MT::Plugin::ObjectGroups;
use strict;
use MT;
use MT::Plugin;
use ItemSort::SortGroup;
use ItemSort::SortNum;
use ItemGroup::ItemGroup;
use ItemGroup::ItemOrder;
use ObjectGroup::ObjectGroup;
use ObjectGroup::ObjectOrder;

our $VERSION = '0.1';

use base qw( MT::Plugin );

@MT::Plugin::ObjectGroups::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::ObjectGroups( {
    id  => 'ObjectGroups',
    key => 'ObjectGroups',
    description => 'Entry,Category,Blog and Object Groups.',
    name => 'ObjectGroups',
    author_name => 'Alfasado,Inc.',
    author_link => 'http://alfasado.net/',
    'version' => $VERSION,
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        tags => {
            block => {
                'EntryGroups' => \&_hndr_object_groups,
                'CategoryGroups' => \&_hndr_object_groups,
                'BlogGroups' => \&_hndr_object_groups,
                'ObjectGroups' => \&_hndr_object_groups,
                'EntryGroupsHeader' => \&_pass_tokens,
                'EntryGroupsFooter' => \&_pass_tokens,
                'CategoryGroupsHeader' => \&_pass_tokens,
                'CategoryGroupsFooter' => \&_pass_tokens,
                'BlogGroupsHeader' => \&_pass_tokens,
                'BlogGroupsFooter' => \&_pass_tokens,
                'ObjectGroupsHeader' => \&_pass_tokens,
                'ObjectGroupsFooter' => \&_pass_tokens,
            },
            function => {
                'GroupName' => \&_hdlr_group_name,
                'ItemCount' => \&_hdlr_item_count,
            },
        }
   } );
}

sub _hndr_object_groups {
    my ( $ctx, $args, $cond ) = @_;
    my $tag = $ctx->stash( 'tag' );
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $blog = $ctx->stash( 'blog' );
    my $blog_ids = $args->{ blog_ids };
    my $blog_id = $args->{ blog_id };
    my $lastn = $args->{ lastn };
    $lastn = 100 unless $lastn;
    my $sort_by = $args->{ sort_by };
    $sort_by = 'id' unless $sort_by;
    my $sort_order = $args->{ sort_order };
    my $direction = 'ascend';
    $direction = $sort_order if $sort_order;
    my $eq = $args->{ eq };
    my $blog_ctx;
    if ( $blog_ids ) {
        my @bids = split( /,/, $blog_ids );
        $blog_id = [ @bids ];
    } else {
        unless ( $blog_id ) {
            $blog_ctx = $blog;
            $blog_id = $blog->id;
        } else {
            if ( $blog_id ne '*' ) {
                if ( $blog_id != $blog->id ) {
                    $blog_ctx = MT::Blog->load( $blog_id ) if ( $blog_id =~ m/[0-9]{1,}/ );
                }
            }
        }
    }
    my $terms;
    if ( $blog_id ne '*' ) {
        unless ( $tag =~ m/blog/i ) {
            $terms->{ blog_id } = $blog_id;
        }
    }
    my $obj_class;
    my $classname;
    if ( $tag =~ m/entry/i ) {
        $obj_class = 'ItemSort::SortGroup';
        $classname = 'entry';
    } elsif ( ( $tag =~ m/category/i ) || ( $tag =~ m/blog/i ) ){
        $obj_class = 'ItemGroup::ItemGroup';
        if ( $tag =~ m/category/i ) {
            $terms->{ object_ds } = 'category';
            $classname = 'category';
        } elsif ( $tag =~ m/blog/i ) {
            $terms->{ object_ds } = 'blog';
            $classname = 'blog';
        }
    } else {
        $obj_class = 'ObjectGroup::ObjectGroup';
        $classname = 'object';
    }
    my @groups = $obj_class->load( $terms,
                           { sort => $sort_by,
                             limit => $lastn,
                             direction => $direction,
                           } );
    my $res = '';
    my $i = 0;
    for my $group ( @groups ) {
        local $ctx->{ __stash }{ classname } = $classname;
        local $ctx->{ __stash }{ blog } = $blog_ctx if ( defined $blog_ctx );
        local $ctx->{ __stash }{ group } = $group;
        my $out = $builder->build( $ctx, $tokens, {
            %$cond,
            EntryGroupsHeader => $i == 0,
            EntryGroupsFooter => !defined( $groups[$i + 1] ),
            CategoryGroupsHeader => $i == 0,
            CategoryGroupsFooter => !defined( $groups[$i + 1] ),
            BlogGroupsHeader => $i == 0,
            BlogGroupsFooter => !defined( $groups[$i + 1] ),
            ObjectGroupsHeader => $i == 0,
            ObjectGroupsFooter => !defined( $groups[$i + 1] ),
        } );
        if ( !defined( $out ) ) { return $ctx->error( $builder->errstr ) };
        $res .= $out;
        $i++;
    }
    $res;
}

sub _hdlr_group_name {
    my ( $ctx, $args, $cond ) = @_;
    my $group = $ctx->stash( 'group' );
    return $group->name || '';
}

sub _hdlr_item_count {
    my ( $ctx, $args, $cond ) = @_;
    my $classname = $ctx->stash( 'classname' );
    my $group = $ctx->stash( 'group' );
    my $group_id = $group->id;
    my $count;
    if ( $classname eq 'entry' ) {
        $count = ItemSort::SortNum->count( { sortgroup_id => $group_id } );
    } elsif ( ( $classname eq 'category' ) || ( $classname eq 'blog' ) ) {
        $count = ItemGroup::ItemOrder->count( { itemgroup_id => $group_id } );
    } else {
        $count = ObjectGroup::ObjectOrder->count( { objectgroup_id => $group_id } );
    }
    return $count if $count;
    return 0;
}

sub _pass_tokens {
    my ( $ctx, $args, $cond ) = @_;
    $ctx->stash( 'builder' )->build($ctx, $ctx->stash( 'tokens' ), $cond );
}

1;